(function(){
    angular
        .module("clockRain")
        .directive("dirRain", dirRain);

    function dirRain(){
        return {
            link: function(scope, elem){
                $(document).ready(function(){
                    var canvas = document.getElementById("night");
                    canvas.width = document.body.clientWidth;
                    canvas.height = window.innerHeight - 75;
                    var rains = canvas.width * canvas.height;
                    window.onresize = function(){
                        canvas.width = document.body.clientWidth;
                        canvas.height = window.innerHeight - 75;
                        rains = canvas.width * canvas.height;
                        fullReset();
                    };
                    var context = canvas.getContext('2d');
                    var rainDropGroups = [];

                    var raining = [];
                    var rainTimeouts = [];
                    makeRainGroup();
                    go();
                    lampFlicker();
                    fadeIn();

                    $(window).on("blur focus", function(e) {
                        var prevType = $(this).data("prevType");

                        if (prevType != e.type) {
                            switch (e.type) {
                                case "blur":
                                    clearRain();
                                    break;
                                case "focus":
                                    makeRainGroup();
                                    go();
                                    break;
                            }
                        }

                        $(this).data("prevType", e.type);
                    });

                    //Making Rain
                    function RainDrop(width){
                        this.x = Math.floor(Math.random() * width);
                        this.y = 0;
                        this.size = Math.round((Math.random() * 25) + 5);
                        this.drop = (Math.floor((Math.random() * 2) + 12));
                        this.drop += this.drop * this.size/14;
                        this.d = (Math.floor(Math.random() * 2) -1);
                        this.delay = Math.floor(Math.random() * 6000);
                    }
                    function makeRainGroup (){
                        rainDropGroups = [];
                        var group = [];
                        for(var i = 0; i < rains/2800; i++){
                            group.push(new RainDrop(canvas.width))
                        }
                        rainDropGroups.push(group);
                    }

                    //Start Rain
                    function go(){
                        rainDropGroups.forEach(function(elem, index, array){
                            elem.forEach(function(element){
                                rainTimeouts.push(setTimeout(function(){
                                    rain(element);
                                }, element.delay))
                            });
                        });

                        function rain(element) {
                            raining.push(setInterval(function(){
                                var oldElement = {
                                    x: element.x,
                                    y: element.y
                                };
                                element.x -= element.d;
                                element.y += element.drop;
                                if( element.x < 0 || element.x > canvas.width || element.y > canvas.height){
                                    element.x = Math.floor(Math.random() * canvas.width);
                                    element.y = 0;
                                }
                                drawRain(element, oldElement);
                            }, 30));

                        }
                        function drawRain(raindrop, oldElement){
                            context.fillStyle = "HSL(33, 38%, 7%)";
                            context.fillRect(oldElement.x - 1, oldElement.y - 1, 4, raindrop.size + 2);
                            var grd = context.createRadialGradient(250, canvas.height - 300, 140, 250, canvas.height - 400, 650);
                            grd.addColorStop(0, 'HSL(33, 38%, 30%)');
                            grd.addColorStop(0.1, 'HSL(33, 38%, 21%)');
                            grd.addColorStop(0.2, 'HSL(33, 38%, 15%)');
                            grd.addColorStop(1, 'HSL(33, 38%, 14%)');
                            context.fillStyle = grd;
                            context.fillRect(raindrop.x, raindrop.y, 2, raindrop.size);
                        }
                    }

                    //Resetting Rain
                    function clearRain(){
                        for(var i = 0; i < raining.length; i++){
                            clearInterval(raining[i]);
                        }
                        for(i = 0; i < rainTimeouts.length; i++){
                            clearTimeout(rainTimeouts[i]);
                        }
                        context.fillStyle = "HSL(33, 38%, 7%)";
                        context.fillRect(0, 0, canvas.width, canvas.height);
                    }
                    function fullReset(){
                        clearRain();
                        makeRainGroup();
                        go();
                    }

                    //Lamp Flicker Animation
                    function lampFlicker(){
                        $(".light").fadeTo(100, .01, function(){
                            $(".light").fadeTo(100, .03)
                        });
                        $(".glass").fadeTo(100, .5, function(){
                            $(".glass").fadeTo(100,.8, function(){
                                setTimeout(lampFlicker, (Math.random() * 5000 + 2000))
                            })
                        });
                    }

                    //Fade Clock In;
                    function fadeIn(){
                        $("#clock").fadeTo(6000,.3)
                    }
                });
            }
        }
    }
})();